<?php

namespace Drupal\sdc_styleguide\Drush\Generators;

use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Serialization\Yaml;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Template\Attribute;
use Drupal\Core\Theme\ComponentPluginManager;
use DrupalCodeGenerator\Asset\AssetCollection;
use DrupalCodeGenerator\Attribute\Generator;
use DrupalCodeGenerator\Command\BaseGenerator;
use DrupalCodeGenerator\GeneratorType;
use Drush\Commands\AutowireTrait;
use Symfony\Component\DependencyInjection\Attribute\Autowire;

/**
 * Generates a custom demo for one of the available SDCs on site.
 */
#[Generator(
  name: 'sdc_styleguide:demo',
  description: 'Generates a SDC Styleguide demo',
  aliases: ['sdcs-demo'],
  templatePath: __DIR__,
  type: GeneratorType::MODULE_COMPONENT,
)]
class SDCStyleguideDemoGenerator extends BaseGenerator {

  use AutowireTrait;
  use StringTranslationTrait;

  /**
   * Constructs a new SDCStyleguideDemoGenerator object.
   */
  public function __construct(
    #[Autowire(service: 'plugin.manager.sdc')]
    protected readonly ComponentPluginManager $componentPluginManager,
    protected readonly FileSystemInterface $fileSystem,
  ) {
    parent::__construct();
  }

  /**
   * {@inheritdoc}
   */
  protected function generate(array &$vars, AssetCollection $assets): void {
    $ir = $this->createInterviewer($vars);
    $definitions = $this->componentPluginManager->getAllComponents();
    $components = [];
    $componentOptions = [];
    foreach ($definitions as $componentDefinition) {
      $definition = $componentDefinition->getPluginDefinition();
      $option = "{$definition['name']} ({$componentDefinition->getPluginId()})";
      $components[$option] = [
        'name' => $definition['name'],
        'machineName' => $definition['machineName'],
        'path' => $definition['path'],
        'properties' => $definition['props']['properties'] ?? [],
        'required' => $definition['props']['required'] ?? NULL,
        'slots' => $definition['slots'] ?? NULL,
        'type' => $definition['extension_type'],
      ];
      $componentOptions[] = $option;
    }
    sort($componentOptions);

    // Get the component to build the demo for.
    $choice = $ir->choice($this->t('For what SDC do you want to create a demo?'), $componentOptions);
    $selectedComponent = $components[$componentOptions[$choice]];
    $this->io()->title($this->t('New demo for @demo_name', [
      '@demo_name' => $selectedComponent['name'],
    ]));

    // Forces a demo name.
    $demoFilename = NULL;
    $demoMachineName = NULL;
    $readName = NULL;
    $ir->ask($this->t('How would you like to name your demo?'), NULL, function ($name) use ($selectedComponent, &$demoMachineName, &$demoFilename, &$readName) {
      if (!$name) {
        throw new \Exception('Please set a value.');
      }

      // Confirms a demo with the same name on component folder does not exist.
      $readName = $name;
      $demoMachineName = preg_replace('/[^a-z0-9]/', '_', strtolower($name));
      $demoFilename = "{$selectedComponent['path']}/{$selectedComponent['machineName']}.demo.{$demoMachineName}.yml";
      if (file_exists($demoFilename)) {
        throw new \Exception('A demo with that name already exists. Please use a different name.');
      }
    });

    // Prepares demo general structure.
    $demo = [
      'name' => $readName,
      'description' => $ir->ask($this->t('Please add a description for your demo. (Optional)')) ?? '',
      'props' => [],
      'slots' => [],
    ];

    // Fills property values.
    if (isset($selectedComponent['properties'])) {
      foreach ($selectedComponent['properties'] as $name => $property) {
        $required = $selectedComponent['required'] ?? [];
        if ($is_attribute = ($property['type'] == Attribute::class)) {
          $property['description'] .= ' For attributes, you need to state them like ATTR_1=VALUE_1;;;ATTR_2=VALUE_2;;;ATTR_3=VALUE_3';
        }

        $demo['props'][$name] = $ir->ask("Please set the {$property['title']} value. ({$property['description']})", NULL, function ($value) use ($name, $required, $is_attribute) {
          if (in_array($name, $required) && !$value) {
            throw new \Exception('This property is required. Please set a value.');
          }

          // Stores attributes in our custom format.
          if ($is_attribute) {
            $value = str_replace(';;;', PHP_EOL, $value);
          }

          // @todo Check for type validation. (bool, number, string, attributes).
          return $value ?? '';
        });
      }
    }

    // Fills demos.
    if (isset($selectedComponent['slots'])) {
      foreach ($selectedComponent['slots'] as $name => $slot) {
        // @todo Ask the user if they want to use Drupal stuff (Nodes, Media, Views, Another SDC Demo) or if they want
        // to use a free form string.
        $demo['slots'][$name] = $ir->ask("Please set the {$slot['title']} value. ({$slot['description']})") ?? '';
      }
    }

    // Writes file.
    $this->fileSystem->saveData(Yaml::encode($demo), $demoFilename, FileSystemInterface::EXISTS_ERROR);
  }

}
